#!/bin/bash
set -x

test_result=
revision=
source_branch=
merge_proposal=
buildjob_url=
TOOLS_PATH='/var/lib/jenkins/ps-qa-tools/scripts/launchpad-trigger'

while getopts "r:v:s:m:b:t:" opt; do
    case $opt in
    r)
        echo "test (r)esult: $OPTARG" >&2
        test_result=$OPTARG
        ;;
    v)
        echo "re(v)ision: $OPTARG" >&2
        revision=$OPTARG
        ;;
    s)
        echo "(s)ource branch: $OPTARG" >&2
        source_branch=$OPTARG
        ;;
    m)
        echo "(m)erge proposal: $OPTARG" >&2
        merge_proposal=$OPTARG
        ;;
    b)
        echo "(b)uild job url: $OPTARG" >&2
        buildjob_url=$OPTARG
        ;;
    t)
        echo "(t)ool-set path: $OPTARG" >&2
        TOOLS_PATH=$OPTARG
        ;;

    \p)
        echo "Invalid option: -$OPTARG" >&2
        exit 1
        ;;
    :)
        echo "Option -$OPTARG requires an argument." >&2
        exit 1
        ;;
    esac
done


if [ "$test_result" == "FAILED" ]; then
    $TOOLS_PATH/updateMergeProposal.py --branch="${source_branch}" \
            --merge-proposal="${merge_proposal}" --job-url="${buildjob_url}"\
            --landing-failed --revision=${revision}

    $TOOLS_PATH/voteOnMergeProposal.py --status='FAILED' \
		--build-url="$BUILD_URL" --branch="$source_branch" \
		--merge-proposal="$merge_proposal" --revision="$revision"
    exit 1
fi

workdir="$WORKSPACE/merging"


result=0

rm -rf "$workdir"
mkdir -p "$workdir"

bzr branch "$target_branch" "$workdir/trunk" || result=1
cd "$workdir/trunk"
bzr merge "$source_branch" || result=1

if [ "$result" == 1 ]; then
    echo "Merging failed"
    $TOOLS_PATH/updateMergeProposal.py --branch="${source_branch}" \
            --merge-proposal="${merge_proposal}" --job-url="${BUILD_URL}" \
	    --merge-failed --revision="${revision}"
    $TOOLS_PATH/voteOnMergeProposal.py --status='FAILED' \
		--build-url="$BUILD_URL" --branch="$source_branch" \
		--merge-proposal="$merge_proposal" --revision="$revision"
    exit 1
fi

TMP_FILE=`mktemp`

#the following code should set the following variables
#commit_message=
#bugs_fixed=
#approved_by=
#launchpad_user=
$TOOLS_PATH/getMergeProposalAttributes.py --merge-proposal="${merge_proposal}" \
        --branch=${source_branch} | grep -v '^DEBUG' > "$TMP_FILE"
. "$TMP_FILE"
cat "$TMP_FILE"
rm -f "$TMP_FILE"

#TODO
#bzr commit --author "" --fixes "" --message "<commit_message>. Fixes: <bugs_fixed>. Approved by <approved_by>."
echo "$authors" "$bugs_fixed_ids" | xargs bzr commit --message "$commit_message. Fixes: $bugs_fixed. Approved by $approved_by." || result=1

if [ -z "$commit_message" ]; then
    echo "Commit message empty."
    $TOOLS_PATH/updateMergeProposal.py --branch="${source_branch}" \
            --merge-proposal="${merge_proposal}" --job-url="${BUILD_URL}"\
            --commit-message-empty --revision="${revision}"

    $TOOLS_PATH/voteOnMergeProposal.py --status='FAILED' \
		--build-url="$BUILD_URL" --branch="$source_branch" \
		--merge-proposal="$merge_proposal" --revision="$revision"
    exit 1

fi

target_branch=`echo "$target_branch" | sed "s,^lp:,bzr+ssh://$launchpad_user@bazaar.launchpad.net/+branch/,"`
bzr push "$target_branch" || result=1

if [ "$result" == 1 ]; then
    echo "Pushing failed"
    $TOOLS_PATH/updateMergeProposal.py --branch="${source_branch}" \
            --merge-proposal="${merge_proposal}" --job-url="${BUILD_URL}"\
            --push-failed --revision="${revision}"
    $TOOLS_PATH/voteOnMergeProposal.py --status='FAILED' \
		--build-url="$BUILD_URL" --branch="$source_branch" \
		--merge-proposal="$merge_proposal" --revision="$revision"
    exit 1
fi

rm -rf "$workdir"

#TODO
$TOOLS_PATH/updateMergeProposal.py --branch="${source_branch}"\
        --merge-proposal="${merge_proposal}" --merged \
        --revision="${revision}"

$TOOLS_PATH/voteOnMergeProposal.py --status='PASSED' \
	--build-url="$BUILD_URL" --branch="$source_branch" \
	--merge-proposal="$merge_proposal" --revision="$revision"

$TOOLS_PATH/closeBugs.py  --branch="$source_branch" \
	--merge-proposal="$merge_proposal" 

