#!/usr/bin/env python

import getopt, sys
from settings import *
import re
from lockfile import FileLock
from launchpad import LaunchpadTrigger


def usage():
    print """usage:
             launchpadTrigger.py -wa
            """

def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "awnb:tj:e:l",['approved','wip','needs-review','branch=','trigger','job=','jenkins-url=','autoland'])
    except getopt.GetoptError, err:
        print str(err)
        usage()
        sys.exit(2)

    state = []
    branch = None
    jenkins_job = None
    trigger_jenkins = False
    trigger_autolanding = False
    jenkins_url = JENKINS_URL
    for o, a in opts:
        if o in ('-a','--approved'):
            state.append('Approved')
        if o in ('-l','--autoland'):
            trigger_autolanding = True
        elif o in ('-w','--wip'):
            state.append('Work in progress')
        elif o in ('-n','--needs-review'):
            state.append('Needs review')
        elif o in ('-b','--branch'):
             branch = a
        elif o in ('-j','--job'):
             jenkins_job = a
        elif o in ('-t','--trigger'):
             trigger_jenkins = True
        elif o in ('-e','--jenkins-url'):
             jenkins_url = a
        else:
            assert False, "unhandled option"

    launchpadTrigger = LaunchpadTrigger()

    if (not trigger_autolanding) and (len(state) == 0):
        print 'Error: you must specify either -a or -w'
        usage()
        sys.exit(2)

    if not branch:
        print 'Error: you must specify branch'
        usage()
        sys.exit(2)

    if (trigger_jenkins or trigger_autolanding) and not jenkins_job:
        print 'Error: you need to define jenkins job if you want to trigger one'
        usage()
        sys.exit(2)

    if (trigger_jenkins or trigger_autolanding) and not jenkins_url:
        print 'Error: you need to define jenkins url if you want to trigger job'
        usage()
        sys.exit(2)

    if trigger_jenkins:
        mps = launchpadTrigger.get_merge_proposals(branch, state)
        launchpadTrigger.trigger_ci_building(mps, jenkins_job, jenkins_url)
        sys.exit(1)

    if trigger_autolanding:
        mps = launchpadTrigger.get_merge_proposals(branch, ['Approved'])
        launchpadTrigger.trigger_al_building(mps, jenkins_job, jenkins_url)
        sys.exit(1)

if __name__ == "__main__":
        print 'DEBUG: going to acquire launchpad lock'
        with FileLock(LOCK_FILE): 
            print 'DEBUG: lock acquired'
            main()
